;;; planner-diary.el --- Diary integration for the Emacs Planner (planner.el)

;; Copyright (C) 2003 Thomas Gehrlein <Thomas.Gehrlein@t-online.de>

;; Emacs Lisp Archive Entry
;; Filename: planner-diary.el
;; Version: $Version$
;; Keywords: hypermedia
;; Author: Thomas Gehrlein <Thomas.Gehrlein@t-online.de>
;; Maintainer: Thomas Gehrlein <Thomas.Gehrlein@t-online.de>
;; Description: Integrate the Emacs Planner with Calendar and Diary
;; URL: http://sacha.free.net.ph/notebook/emacs/planner/planner-diary.el
;; ChangeLog: Can be requested from the maintainer
;; Compatibility: Emacs20, Emacs21

;; This file is not part of GNU Emacs.

;; This is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2, or (at your option) any later
;; version.
;;
;; This is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
;; MA 02111-1307, USA.

;;; Commentary:
;;
;; Place planner-diary.el in your load path and add this to your .emacs:
;;
;;    (require 'planner-diary)
;;
;; planner-diary.el can either display or rewrite a diary section into your
;; planner file. If you want it to automatically rewrite your present and
;; future pages, add
;;
;;    (planner-diary-insinuate)
;;
;; to your .emacs . Whenever you visit a day page, the contents of *
;; Diary will be replaced by entries from your ~/.diary. Changes in
;; this section will NOT be saved. Only pages >= today are
;; modified--pages in the past are NOT automatically rewritten. You
;; can force an update with M-x planner-diary-insert-diary (works with
;; pages in the past as well) or M-x planner-diary-insert-diary-maybe
;; (present and future only). You may want to bind
;; `planner-diary-insert-diary-maybe' to a key.
;;
;; If you want to view diary entries from your planner page without
;; performing rewrites, do not call (planner-diary-insinuate).
;; Instead, add the following to your `planner-day-page-template':
;;
;; <lisp>(planner-diary-entries-here)</lisp>
;;
;; This will be marked up, but not rewritten. Days in the past will
;; reflect the current ~/.diary. Note that some font lock support
;; modes postpone evaluating like this. If the <lisp> code is not
;; getting marked up, try M-x emacs-wiki-highlight-buffer . If it
;; works, then try
;;
;;    (setq font-lock-support-mode nil)                        or
;;    (setq font-lock-support-mode '((planner-mode . nil)
;;                                   (t . jit-lock-mode)))
;;
;; If you want to see your diary entries for more than just 1 day, set
;; `planner-diary-number-of-diary-entries' accordingly.

;; Thanks to:
;;
;; - Jody Klymak (jklymak AT coas DOT oregonstate DOT edu)
;;   Added some documentation
;;
;; - Sacha Chua (sacha AT free DOT net DOT ph)
;;   Thoughts and ideas

;;; Code:

(require 'planner)
(require 'diary-lib)
(require 'calendar)

(defcustom planner-diary-update-automatically t
  "If non-nil, update the `planner-diary-string' section automatically.
Only present or future day pages are updated."
  :type 'string
  :group 'planner)

(defcustom planner-diary-string "* Diary"
  "*Header for the diary section in a day plan page."
  :type 'string
  :group 'planner)

(defcustom planner-diary-number-of-diary-entries 1
  "*Specifies how many days of diary entries are displayed."
  :type 'number
  :group 'planner)

;;; functions
(defun planner-diary-get-diary-entries (date &optional no-of-days)
  "For DATE, get the NO-OF-DAYS diary entries for DATE and the following days.
DATE is a list (month day year).  NO-OF-DAYS defaults to
`planner-diary-number-of-diary-entries'"
  (save-window-excursion
    (save-excursion
      (let* ((fancy-diary-buffer "*temporary-fancy-diary-buffer*")
             (entries)
             (no-of-days (or no-of-days planner-diary-number-of-diary-entries)))
        ;; no messages from list-diary-entries
        (let ((diary-display-hook 'fancy-diary-display))
          (list-diary-entries date no-of-days)
          (switch-to-buffer fancy-diary-buffer)
          ;; return empty string if buffer is empty
          (if (= (point-max) 1)
              (setq entries "")
            (setq entries
                  (buffer-substring
                   (if (> no-of-days 1)   ; if more than 1 day, show everything
                       (point-min)
                     ;; remove date and lots of =s if just for 1 day
                     (goto-char (point-min))
                     (search-forward-regexp "^=+$") ; one or more =
                     (1+ (point)))
                   ;; remove final newline
                   (progn (goto-char (point-max))
                          (when (bolp) (backward-char 1))
                          (point)))))
          (kill-buffer fancy-diary-buffer)
          entries)))))

;;; If you don't want to rewrite your pages, you can put this is your
;;; day pages: <lisp>(planner-diary-entries-here)</lisp>
(defun planner-diary-entries-here ()
  "Display the diary for the current planner day pages.
You might want to use `planner-day-page-template'."
  (planner-diary-get-diary-entries
   (planner-filename-to-calendar-date (emacs-wiki-page-name))))

(defun planner-diary-insert-diary (&optional force)
  "Insert the fancy diary for the day into the day plan file.
If FORCE is non-nil, insert `planner-diary-string' at the beginning of the
buffer if it is not there yet."
  ;; search for "^* Diary$", delete buffer content to the next "^* ", insert
  (interactive "P") ; raw prefix arg
  ;; sanity checks
  (unless (and (equal major-mode 'planner-mode)
               (string-match planner-date-regexp (buffer-name)))
    (error "Cannot insert diary in this buffer"))
  (save-excursion
    (goto-char (point-min))
    (or (re-search-forward (concat "^" planner-diary-string "$") (point-max) t)
        (when force
          (insert planner-diary-string "\n\n\n")
          (backward-char 3)
          t)
        (error "Cannot insert diary, no diary section in this buffer"))
    ;; point is at the end of "* Diary"
    ;; (forward-char 1)
    (let ((beg (point))
          (end (if (re-search-forward "^* " (point-max) t)
                   (progn (beginning-of-line)
                          (backward-char 1)
                          (point))
                 (point-max))))
      (delete-region beg end)
      ;; point is at the end of "* Diary"
      (let ((entries (planner-diary-get-diary-entries
                      (planner-filename-to-calendar-date (buffer-name)))))
        (insert "\n\n")
        (unless (string= entries "")
          (insert entries
                  "\n"))))))

(defun planner-diary-insert-diary-maybe (&optional force)
  "Maybe insert the fancy diary for the day into the day plan file.
If the current day is in the past, don't do anything.
If FORCE is non-nil, insert `planner-diary-string' at the beginning of the
buffer if it is not there yet."
  (interactive "P")
  ;; check if we're in a day plan buffer
  (when (string-match planner-date-regexp (buffer-name))
    (if (string< (buffer-name) (planner-today))
        ;; we are in the past -> do nothing, message when interactive
        (when (interactive-p)
          (message "Date is in the past.  No diary entries inserted."))
      ;; today or future
      (condition-case nil
          (planner-diary-insert-diary force)
        (error nil)))))

(defun planner-diary-update-diary ()
  "Function suitable for adding to `planner-mode-hook'."
  (when planner-diary-update-automatically
    (planner-diary-insert-diary-maybe)))

(defun planner-diary-show-day-plan-or-diary ()
  "Show the day plan or diary entries for the date under point in calendar.
Add this to `calendar-move-hook' if you want to use it.  In that case you
should also `remove-hook' `planner-calendar-show' from `calendar-move-hook'."
  (interactive)
  (or (planner-calendar-show)
      (view-diary-entries 1)))

(defun planner-diary-setup ()
  "Add automatic planner updates to the appropriate hooks."
  (planner-diary-update-diary)
  (add-hook (if (boundp 'write-file-functions)
                'write-file-functions
              'local-write-file-hooks)
            'planner-diary-update-diary t))

(defun planner-diary-insinuate ()
  "Hook Diary into Planner.
Automatically insert and update a Diary section in day plan files."
;;  (define-key planner-mode-map "\C-c\C-e" 'planner-diary-insert-diary-maybe)
  (add-hook 'planner-mode-hook 'planner-diary-setup))

(defalias 'planner-insinuate-diary 'planner-diary-insinuate)

(provide 'planner-diary)

;;; planner-diary.el ends here
